/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.uinfo.salix;

import java.util.ArrayList;
import java.util.Collection;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.IExportContext;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;
import org.dive4elements.river.artifacts.sinfo.util.WstInfo;
import org.dive4elements.river.artifacts.uinfo.commons.UInfoResultType;

/**
 * @author Domenico Nardi Tironi
 */
final class SalixLineCalculationResult extends AbstractCalculationExportableResult {

    private static final long serialVersionUID = 1L;
    private static final String JASPER_FILE = "/jasper/templates/uinfo.salixline.jrxml";

    public SalixLineCalculationResult(final String label, final WstInfo wst, final Collection<ResultRow> rows) {
        super(label, rows);
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final Collection<String> header = new ArrayList<>(4);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        // header.add(msgUnit(SInfoResultType.flowdepth, SInfoResultType.flowdepth.getUnit()));

        header.add(exportContextCSV.formatCsvHeader(UInfoResultType.salixline));
        // wenn "historisch" gewählt wurde, nur "historisch" anzeigen; sonst für jeden scen-wert ne neue Spalte und "hist"
        // ausblenden!...!..!!
        header.add(exportContextCSV.formatCsvHeader(UInfoResultType.salixlinehist));
        header.add(exportContextCSV.formatCsvHeader(UInfoResultType.salixlinescen));
        exportContextCSV.writeCSVLine(header.toArray(new String[header.size()]));

    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        writeRegionalEffectsCSVMetadata(exportContextCSV);
        exportContextCSV.writeBlankLine();
        writeExtendedRegionalEffectsCSVMetadata(exportContextCSV);
        exportContextCSV.writeBlankLine();
        writeHistoricalViewCSVMetadata(exportContextCSV);
        exportContextCSV.writeBlankLine();

    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {

        return formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextPDF, final ResultRow row) {

        return formatRow(exportContextPDF, row);
    }

    @Override
    protected String getJasperFile() {

        return this.JASPER_FILE;
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContextPDF, final MetaAndTableJRDataSource source) {

        /* additional column headings */
        exportContextPDF.addJRMetadata(source, "station_header", GeneralResultType.station);

        exportContextPDF.addJRMetadata(source, "salix_line", UInfoResultType.salixline);
        // wenn "historisch" gewählt wurde, nur "historisch" anzeigen; sonst für jeden scen-wert ne neue Spalte und "hist"
        // ausblenden!...!..!!
        exportContextPDF.addJRMetadata(source, "salix_line_hist", UInfoResultType.salixlinehist);
        exportContextPDF.addJRMetadata(source, "salix_line_scen", UInfoResultType.salixlinescen);

    }

    private String[] formatRow(final IExportContext context, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(3);

        lines.add(context.formatRowValue(row, GeneralResultType.station));
        lines.add(context.formatRowValue(row, UInfoResultType.salixline));

        // wenn "historisch" gewählt wurde, nur "historisch" anzeigen; sonst für jeden scen-wert ne neue Spalte und "hist"
        // ausblenden!...!..!!
        lines.add(context.formatRowValue(row, UInfoResultType.salixlinehist));
        lines.add(context.formatRowValue(row, UInfoResultType.salixlinescen));
        return lines.toArray(new String[lines.size()]);
    }

    private void writeRegionalEffectsCSVMetadata(final ExportContextCSV exportContextCSV) {
        final String main = "uinfo.export.csv.meta.header.salix.regional";
        // "##Regional wirkende Eingriffe"
        exportContextCSV.writeCSVMetaEntry(main);
        writeRegionalCommonCSVMeta(exportContextCSV);
    }

    private void writeRegionalCommonCSVMeta(final ExportContextCSV exportContextCSV) {
        final String main = "uinfo.export.csv.meta.header.salix";
        // "# Szenariotyp: "
        exportContextCSV.writeCSVMetaEntry(main + ".szenariotyp");
        // "# Teilabschnitt: "
        exportContextCSV.writeCSVMetaEntry(main + ".teilabschnitt");
        // "# Mittelwasserspiegellagenänderung: "
        exportContextCSV.writeCSVMetaEntry(main + ".mwspiegellaenderung");
    }

    private void writeExtendedRegionalEffectsCSVMetadata(final ExportContextCSV exportContextCSV) {
        final String main = "uinfo.export.csv.meta.header.salix.regionalextended";
        // "##Überregional wirkende Eingriffe"
        exportContextCSV.writeCSVMetaEntry(main);

        writeRegionalCommonCSVMeta(exportContextCSV);
    }

    private void writeHistoricalViewCSVMetadata(final ExportContextCSV exportContextCSV) {
        final String main = "uinfo.export.csv.meta.header.salix.historical";
        // "##Historische Betrachtung"
        exportContextCSV.writeCSVMetaEntry(main);

        final String mainSub = "uinfo.export.csv.meta.header.salix";
        // "# Szenariotyp: "
        exportContextCSV.writeCSVMetaEntry(mainSub + ".szenariotyp");
        // "# Teilabschnitt: "
        exportContextCSV.writeCSVMetaEntry(mainSub + ".teilabschnitt");

        // "# Art des Zeitraums: "
        exportContextCSV.writeCSVMetaEntry(main + ".zeitart");
        // "# Historischer Zeitpunkt: "
        exportContextCSV.writeCSVMetaEntry(main + ".zeitpunkt");
    }
}